/**
 * @file 	MultiInterval.java
 * 
 * @date 	Jan 15, 2005
 * @author 	Alexandre Bouchard
 */

package ca.mcgill.lmp.util;

import java.util.Random;

import ca.mcgill.lmp.EmptySetException;
import ca.mcgill.lmp.MeasurableSet;
import ca.mcgill.lmp.State;

/**
 *
 */
public class MultiInterval implements MeasurableSet
{
    private double [] lowerBounds;
    private double [] upperBounds;
    private boolean [] isClosedLeft;
    private boolean [] isClosedRight;
    private Random rand = new Random(System.currentTimeMillis());
    
    public MultiInterval(double left, double right)
    {
        lowerBounds = new double[1];
        lowerBounds[0] = left;
        upperBounds = new double[1];
        upperBounds[0] = right;
        isClosedLeft = new boolean[1];
        isClosedLeft[0] = true;
        isClosedRight = new boolean[1];
        isClosedRight[0] = true;
    }
    
    public MultiInterval(double left, double right, boolean isClosedLeft, boolean isClosedRight)
    {
        lowerBounds = new double[1];
        lowerBounds[0] = left;
        upperBounds = new double[1];
        upperBounds[0] = right;
        this.isClosedLeft = new boolean[1];
        this.isClosedLeft[0] = isClosedLeft;
        this.isClosedRight = new boolean[1];
        this.isClosedRight[0] = isClosedRight;
    }
    
    public MultiInterval(double [] left, double [] right, boolean [] isClosedLeft, boolean [] isClosedRight)
    {
        lowerBounds = left;
        upperBounds = right;
        this.isClosedLeft = isClosedLeft;
        this.isClosedRight = isClosedRight;
    }
    
    public MultiInterval(double [] left, double [] right)
    {
        lowerBounds = left;
        upperBounds = right;
        isClosedLeft = new boolean[left.length];
        isClosedRight = new boolean[left.length];
        for (int i = 0; i < left.length; i++)
        {
            isClosedLeft[i] = true;
            isClosedRight[i] = true;
        }
    }
    
    public MultiInterval(int n)
    {
        lowerBounds = new double[n];
        upperBounds = new double[n];
        isClosedLeft = new boolean[n];
        isClosedRight = new boolean[n];
        for (int i = 0; i < n; i++)
        {
            lowerBounds[i] = 0;
            upperBounds[i] = 1;
            isClosedLeft[i] = true;
            isClosedRight[i] = true;
        }
    }
    
    public int getDimensionality()
    {
        return lowerBounds.length;
    }
    
    public boolean contains(double x)
    {
        if (x > lowerBounds[0] && x < upperBounds[0])
        {
            return true;
        }
        if (x == lowerBounds[0] && isClosedLeft[0])
        {
            return true;
        }
        if (x == upperBounds[0] && isClosedRight[0])
        {
            return true;
        }
        return false;
    }

    public boolean contains(State s)
    {
        if (!(s instanceof RnState) 
                || ((RnState) s).getDimensionality() != this.getDimensionality())
        {
            return false;
        }
        RnState state = (RnState) s;
        for (int i = 0; i < getDimensionality(); i++)
        {
            if (state.getCoordinate(i) > upperBounds[i])
            {
                return false;
            }
            if (state.getCoordinate(i) < lowerBounds[i])
            {
                return false;
            }
            if (state.getCoordinate(i) == lowerBounds[i] && !isClosedLeft[i])
            {
                return false;
            }
            if (state.getCoordinate(i) == upperBounds[i] && !isClosedRight[i])
            {
                return false;
            }
        }
        return true;
    }

    public State sampleUniformly() throws EmptySetException
    {
        if (area() == 0)
        {
            throw new EmptySetException();
        }
        double [] coordinates = new double[getDimensionality()];
        for (int i = 0; i < getDimensionality(); i++)
        {
            double scaling = upperBounds[i] - lowerBounds[i];
            double translation = lowerBounds[i];
            coordinates[i] = rand.nextDouble() * scaling + translation;
        }
        return new RnState(coordinates);
    }
    
    public double area()
    {
        double partialProduct = 1;
        for (int i = 0; i < getDimensionality(); i++)
        {
            partialProduct = partialProduct * (upperBounds[i] - lowerBounds[i]);
        }
        return partialProduct;
    }
    
    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        for (int i = 0; i < getDimensionality(); i++)
        {
            if (isClosedLeft[i])
            {
                buffer.append("[");
            }
            else
            {
                buffer.append("(");
            }
            buffer.append("" + lowerBounds[i] + ", " + upperBounds[i]);
            if (isClosedRight[i])
            {
                buffer.append("]");
            }
            else
            {
                buffer.append(")");
            }
            if (i != getDimensionality() - 1)
            {
                buffer.append(" X ");
            }
        }
        return buffer.toString();
    }
}
